% Load data (assuming 'data' table is already loaded in the workspace)
data.time_in_state = data.exit - data.entry;

disp('States def. (1) admission; (2) ICU; (3) Discharge; (4) Death');

% Create transition matrix
states = unique([data.from; data.to]);
transition_matrix = zeros(length(states));

for i = 1:length(states)
    time_in_state = sum(data.from == states(i)) > 0;
    if time_in_state == 0
        time_in_state = 1;
    else
        time_in_state = sum(data(data.from == states(i), :).time_in_state);
    end
    
    for j = 1:length(states)
        transition_matrix(i, j) = sum((data.from == states(i)) & (data.to == states(j))) / time_in_state;
    end
end

% Adjust diagonal of transition matrix
for i = 1:length(states)
    transition_matrix(i, i) = -sum(transition_matrix(i, :));
end

% Call stacked_probability_plot function
stacked_probability_plot(0, max(data.exit), 1, transition_matrix, 1, [4, 2, 1, 3], false, lines(length(states)));

return

function expmQ = expm_custom(Q)
    % Calculate matrix exponential using eigen decomposition
    [V, D] = eig(Q);
    expD = diag(exp(diag(D)));
    expmQ = V * expD / V;
end

function stacked_probability_plot(Tmin, Tmax, dt, Q, init_state, ord, border, color_scheme)
    % Initialize time interval
    times = Tmin:dt:Tmax;
    
    % Initial state probability vector
    p = zeros(1, size(Q, 1));
    p(init_state) = 1;
    
    % Initialize probability matrix
    P = [];
    
    for t = times
        % Calculate state occupation probabilities
        P = [P; p * expm_custom(Q * t)];
    end
    
    % Reorder probability matrix
    P = P(:, ord);
    
    % Create initial empty plot
    figure;
    hold on;
    xlim([min(times), max(times)]);
    ylim([0, 1]);
    xlabel('time (day)');
    ylabel('Probability');
    
    for state = 1:size(P, 2)
        if state == 1
            acc_1 = P(:, state);
            fill([times, max(times)], [acc_1; 0], color_scheme(2*state+1), 'EdgeColor', 'none');
        else
            acc_2 = acc_1;
            acc_1 = acc_1 + P(:, state);
            fill([times, fliplr(times)], [acc_1; flipud(acc_2)], color_scheme(2*state+1), 'EdgeColor', 'none');
        end
    end
    
    % Add legend to plot
    legend(arrayfun(@(x) strcat('State ', num2str(x)), [4,2,1,3], 'UniformOutput', false), 'Location', 'northeastoutside');
    hold off;
end
